package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.HBJMLeftTreeConstants;

import java.util.Arrays;
import java.util.List;

/**
 * @author rimwood
 * @desc 职位体系方案下族数据库操作Repository
 * @create 2021-12-10 09:37:42
 **/
public class JobFamilyScmRepository {
    private static class JobFamilyScmInstance {
        private static JobFamilyScmRepository INSTANCE = new JobFamilyScmRepository();
    }

    public static JobFamilyScmRepository getInstance() {
        return JobFamilyScmInstance.INSTANCE;
    }

    /**
     * 根据职位体系方案id查询下面的族
     * @param jobScmId 职位体系方案id
     * @return dyns 职位体系方案下的职位族集合
     */
    public DynamicObject[] queryJobFamilyScmByScmId(Long jobScmId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, jobScmId);
        return serviceHelper.query("id,name,number,jobseq,jobfamily,createorg,jobgradescm,joblevelscm",new QFilter[]{scmFilter,enableFilter},"number asc,id desc");
    }

    public DynamicObject[] queryJObFamilyScmByScmId(List<Long> jobScmId,List<Long> jobSeqId, Boolean isShowDisable, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter enableFilter;
        if(isShowDisable){
            enableFilter = new QFilter("jobfamily.enable", QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED,HRBaseConstants.DISABLED}));
        } else {
            enableFilter = new QFilter("jobfamily.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        QFilter scmFilter = new QFilter(HBJMLeftTreeConstants.JOB_SCM, QFilter.in, jobScmId);
        QFilter seqFilter = new QFilter(HBJMLeftTreeConstants.JOB_SEQ, QFilter.in, jobSeqId);
        QFilter initstatusFilter=new QFilter("jobfamily.initstatus",QFilter.not_in,new String[]{"0","1"});
        return serviceHelper.queryOriginalArray("jobfamily.name,jobseq,jobfamily,jobfamily.enable,jobscm",new QFilter[]{scmFilter,enableFilter,seqFilter,initstatusFilter}, orderCondition);
    }

    /**
     * 根据职位体系方案id和职位族查询下面的族
     * @param jobScmId 职位体系方案id
     * @param familyIds 职位族id集合
     * @return dyns 职位体系方案下的职位族集合
     */
    public DynamicObject[] queryJobFamilyScm(Long jobScmId, List<Long> familyIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, jobScmId);
        QFilter familyFilter = new QFilter("jobfamily", QFilter.in, familyIds);
        return serviceHelper.query("id,name,number,jobseq,jobfamily,createorg,jobgradescm,joblevelscm",new QFilter[]{scmFilter,enableFilter,familyFilter},"number asc,id desc");
    }

    /**
     * 批量保存职位序列
     * @param jobFamilyCollection 职位序列集合
     */
    public void saveBatch(DynamicObjectCollection jobFamilyCollection){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        serviceHelper.save(jobFamilyCollection);
    }

    /**
     * 删除职位体系方案关联的职位族
     * @param scmId 职位体系方案Id
     */
    public void deleteByScmId(Long scmId){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, scmId);
        serviceHelper.deleteByFilter(new QFilter[]{scmFilter});
    }

    /**
     * 删除职位体系方案关联的职位族
     * @param scmIds 职位体系方案Id
     */
    public void deleteByScmIds(List<Long> scmIds){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.in, scmIds);
        serviceHelper.deleteByFilter(new QFilter[]{scmFilter});
    }

    /**
     * 根据职位体系方案ID与职位族ID
     *
     * @param jobScmIds    职位体系ID(批量)
     * @param jobFamilyIds 职位族ID(批量)
     * @return 获取职位体系方案族的信息, 后续自行处理
     */
    public DynamicObject[] getJobFamilyScmDyobjsByJobScmIdsAndJobFamilyIds(List<Long> jobScmIds,
                                                                           List<Long> jobFamilyIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter qFilter = new QFilter("jobscm.id", QFilter.in, jobScmIds);
        qFilter.and(new QFilter("jobfamily.id", QFilter.in, jobFamilyIds));

        return serviceHelper.query("jobfamily, jobscm, jobgradescm, joblevelscm", new QFilter[] {qFilter});
    }

    /**
     * 根据职位体系方案编码
     * @param scmnumber  职位体系方案编码
     * @return 职位体系方案族的信息
     */
    public DynamicObject getFamilyScmDyn(String scmnumber) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter enableFilter = new QFilter("jobscm.enable", QFilter.equals, HRBaseConstants.ENABLED);
        QFilter statusFilter = new QFilter("jobscm.status", QFilter.equals, HRBaseConstants.STATUS_AUDIT);
        QFilter scmnumberFilter = new QFilter("jobscm.number", QFilter.equals, scmnumber);
        return serviceHelper.queryOne("jobscm,jobscm.number,jobscm.name,jobseq,jobseq.number,jobseq.name,id,jobfamily.number,jobfamily.name", new QFilter[]{scmnumberFilter, enableFilter,statusFilter});
    }

    /**
     * 根据职位体系方案id和职位族查询下面的族
     * @param jobScmIds 职位体系方案id集合
     * @param familyIds 职位族id集合
     * @return dyns 职位体系方案下的职位族集合
     */
    public DynamicObject[] queryJobFamilyScmByScmIdsAndFamilyIds(List<Long> jobScmIds, List<Long> familyIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.in, jobScmIds);
        QFilter familyFilter = new QFilter("jobfamily", QFilter.in, familyIds);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{scmFilter,familyFilter});
    }

    /**
     * 根据职位体系方案id和职位族查询下面的族
     * @param jobScmIds 职位体系方案id集合
     * @param familyIds 职位族id集合
     * @return dyns 职位体系方案下的职位族集合
     */
    public DynamicObject[] queryJobFamilyScmFieldByScmIdsAndFamilyIds(List<Long> jobScmIds, List<Long> familyIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.in, jobScmIds);
        QFilter familyFilter = new QFilter("jobfamily", QFilter.in, familyIds);
        String cols = "jobscm,jobseq,id,jobfamily, jobgradescm, joblevelscm";
        return serviceHelper.query(cols,new QFilter[]{scmFilter,familyFilter});
    }

}
