package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.util.HRObjectUtils;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author rd_yan_h
 * @desc 职位查询相关的Repository
 * @create 2021/12/06 10:14
 **/
public class JobQueryRepository {
    private static class JobQueryInstance {
        private static JobQueryRepository INSTANCE = new JobQueryRepository();
    }

    public static JobQueryRepository getInstance() {
        return JobQueryInstance.INSTANCE;
    }

    /**
     * 根据职位体系方案id查询职位id
     *
     * @param scmId 职位体系方案id
     * @param filedId 字段id
     * @param filed 字段
     * @param isShowDisable 是否显示禁用
     * @return
     */
    public List<Long> queryJobId(Long scmId, List<Long> filedId, String filed, Boolean isShowDisable) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOB);
        // 当前表过滤
        QFilter currentFilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ONE);
        if (scmId != 0l) {
            currentFilter.and(new QFilter("jobscm.id", QFilter.equals, scmId));
        }
        QFilter filter = new QFilter(filed, QFilter.in, filedId);
        DynamicObject[] dynamicObjects = serviceHelper.queryOriginalArray(HRBaseConstants.ID, new QFilter[]{currentFilter, filter});
        if (HRObjectUtils.isEmpty(dynamicObjects) || dynamicObjects.length == 0) {
            return new ArrayList<>();
        }
        return Arrays.stream(dynamicObjects).map(dy -> dy.getLong(HRBaseConstants.ID)).collect(Collectors.toList());
    }

    /**
     * 根据职位id查询职位信息
     *
     * @param id 职位id
     * @return dy
     */
    public DynamicObject queryJobInfoByID(Long id){
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter(HRBaseConstants.ID, QFilter.equals, id);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(JobConstans.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ONE);
        return serviceHelper.queryOne("id,jobscm,jobfamily,jobclass,lowjoblevel,highjoblevel,lowjobgrade,highjobgrade", new QFilter[]{idFilter,enableFilter,currentFilter});
    }
}
