package kd.hrmp.hbjm.business.domain.repository;

import com.google.common.collect.Sets;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.HRHisBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.EnumHisDataVersionStatus;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hr.hbp.common.util.HRDateTimeUtils;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.util.HashMapUtil;
import kd.hrmp.hbjm.common.util.QFilterHelper;

import java.util.*;

public class JobRepository {
    private static class JobRepositoryInstance {
        private static JobRepository INSTANCE = new JobRepository();
    }

    public static JobRepository getInstance() {
        return JobRepositoryInstance.INSTANCE;
    }

    /**
     * 查询职位数据
     *
     * @param id 主键
     * @return
     */
    public DynamicObject findById(Long id) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        return serviceHelper.queryOne(id);
    }

    /**
     * 根据职位类id集合查询职位中的职位类
     *
     * @param ids 职位类id集合
     * @return 职位类数组
     */
    public DynamicObject[] queryByJobClassIds(Collection<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idsFilter = new QFilter("jobclass", QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.queryOriginalArray("jobclass", new QFilter[]{currentFilter, idsFilter, enableFilter});
    }

    /**
     * 根据职位类id集合查询职位中的职位类
     *
     * @param ids 职位类id集合
     * @return 职位类数组
     */
    public DynamicObject[] queryByJobSeqIds(Collection<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOBFAMILIY);
        QFilter idsFilter = new QFilter("jobseq", QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.queryOriginalArray("jobseq,jobseq.number",
                new QFilter[]{currentFilter, idsFilter, enableFilter});
    }

    /**
     * 根据职位id集合查询职位中的职位体系方案
     *
     * @param ids 职位类id集合
     * @return 职位体系方案数组
     */
    public DynamicObject[] queryByJobScmIds(Collection<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idsFilter = new QFilter("jobscm", QFilter.in, ids);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLED);
        QFilter currentFilter = new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE);
        return serviceHelper.query("id,name,number,jobscm", new QFilter[]{currentFilter, idsFilter, enableFilter});
    }

    /**
     * 查询今天需要生效的职位
     *
     * @return
     */
    public DynamicObject[] findFurtherEffectJob() {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        return serviceHelper.query(new QFilter[]{
                new QFilter(HBJMConstants.ISCURRENTVERSION, "=", HRBaseConstants.STR_ONE),
                new QFilter(HRBaseConstants.ENABLE, "=", HRBaseConstants.ENABLING),
                new QFilter(HRHisBaseConstants.FIELD_BSED, QFilter.less_equals, HRDateTimeUtils.truncateDate(new Date()))});
    }

    /**
     * 查询职位变更历史记录
     *
     * @return
     */
    public DynamicObject[] findJobChangeHis(Long boid) {

        return BusinessDataServiceHelper.load(HBJMConstants.PAGE_HBJM_JOB,
                "id,name,bsed,changedesc,creator",
                new QFilter[]{new QFilter(HBJMConstants.BOID, QFilter.equals, boid),
                        new QFilter(HBJMConstants.ISCURRENTVERSION, QFilter.equals, "0"),
                        new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QFilter.in,
                                new String[]{HRHisBaseConstants.DATA_STATUS_SCHEDULEEFFECT,
                                        HRHisBaseConstants.DATA_STATUS_EFFECTING, HRHisBaseConstants.DATA_STATUS_INVALID,
                                        HRHisBaseConstants.DATA_STATUS_DELETE})},
                "bsed desc,createtime desc");
    }

    /**
     * 查询职位变更历史记录
     *
     * @return
     */
    public DynamicObject[] findJobsChangeHis(Set<Long> boids) {

        return BusinessDataServiceHelper.load(HBJMConstants.PAGE_HBJM_JOB,
                "id,name,bsed,creator,modifier,boid,createtime,modifytime",
                new QFilter[]{new QFilter(HBJMConstants.BOID, QFilter.in, boids),
                        new QFilter(HBJMConstants.ISCURRENTVERSION, QFilter.equals, "0"),
                        new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QFilter.in,
                                new String[]{HRHisBaseConstants.DATA_STATUS_SCHEDULEEFFECT,
                                        HRHisBaseConstants.DATA_STATUS_EFFECTING, HRHisBaseConstants.DATA_STATUS_INVALID,
                                        HRHisBaseConstants.DATA_STATUS_DELETE})},
                "boid asc,bsed desc,modifytime desc");
    }

    /**
     * 找到最近一个历史版本
     *
     * @param boid
     * @return
     */
    public long findLastChangeHisID(Long boid) {
        DynamicObjectCollection collection = QueryServiceHelper.query(HBJMConstants.PAGE_HBJM_JOB, HRBaseConstants.ID,
                new QFilter[]{new QFilter(HBJMConstants.BOID, QFilter.equals, boid),
                        new QFilter(HBJMConstants.ISCURRENTVERSION, QFilter.equals, "0")},
                "bsed desc", 1);
        if (collection.size() > 0)
            return collection.get(0).getLong(HRBaseConstants.ID);
        else
            return 0;
    }

    /**
     * 根据当前生效日期获得其失效日期 1、返回空为未来生效
     *
     * @param bsed
     * @return
     */
    public Date getBlsedByBsed(Date bsed, long boid) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        DynamicObject[] dynamicObjects = serviceHelper.queryOriginalArray(HRHisBaseConstants.FIELD_BSED,
                new QFilter[]{
                        new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QFilter.in,
                                new String[]{HRHisBaseConstants.DATA_STATUS_SCHEDULEEFFECT,
                                        HRHisBaseConstants.DATA_STATUS_EFFECTING, HRHisBaseConstants.DATA_STATUS_INVALID}),
                        new QFilter(HBJMConstants.BOID, QFilter.equals, boid)},
                "bsed asc");
        for (DynamicObject b : dynamicObjects) {
            Date date = b.getDate(HRHisBaseConstants.FIELD_BSED);
            if (bsed.before(date)) {
                return HRDateTimeUtils.addDay(date, -1);
            }
        }
        return null;
    }

    /**
     * 获取当前变更操作受影响的历史版本数据。批量
     *
     * @param bomap        当前变动操作Map<bo-生效日期>
     * @param newHisIdList 当前变更操作最新产生的历史版本
     * @return
     */
    public Map<Long, DynamicObject> getChangeBeforeHisJob(Map<Long, Date> bomap, List<Long> newHisIdList) {
        Map<Long, DynamicObject> result = new HashMap<>(HashMapUtil.getInitCapacity(bomap.size()));
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        DynamicObject[] dynamicObjects = serviceHelper.queryOriginalArray("id,boid,bsed",
                new QFilter[]{new QFilter(HBJMConstants.BOID, QFilter.in, bomap.keySet()),
                        new QFilter(HRBaseConstants.ID, QFilter.not_in, newHisIdList),
                        new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QFilter.equals, "0")},
                "boid desc,bsed desc,id desc");
        Set<Long> set = Sets.newHashSetWithExpectedSize(newHisIdList.size());
        for (DynamicObject his : dynamicObjects) {
            long boid = his.getLong(HBJMConstants.BOID);
            if (set.contains(boid)) {
                continue;
            }
            Date jobbsed = bomap.get(boid);
            Date date = his.getDate(HRHisBaseConstants.FIELD_BSED);
            if (date.after(jobbsed)) {
                continue;
            }
            result.put(boid, his);
            set.add(boid);
        }
        return result;
    }

    /**
     * 获取禁用操作产生的历史版本
     *
     * @param ids
     * @return
     */
    public DynamicObject[] getDisableHis(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, ids);
        QFilter currentversionFilter =
                new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ZERO);
        QFilter datastatusFilter = new QFilter("datastatus", QFilter.equals, HRBaseConstants.STR_ONE);
        QFilter enableFilter = new QFilter("enable", QFilter.equals, HRBaseConstants.DISABLED);
        QFilter initStatusFilter = new QFilter("initstatus", QFilter.not_in, new String[]{"0", "1"});
        return serviceHelper.loadDynamicObjectArray(
                new QFilter[]{idFilter, currentversionFilter, datastatusFilter, enableFilter, initStatusFilter});
    }

    /**
     * 获取禁用操作产生的历史版本
     *
     * @param ids
     * @return
     */
    public DynamicObject[] getEnableHis(Set<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, ids);
        QFilter currentversionFilter =
                new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ZERO);
        QFilter datastatusFilter = new QFilter("datastatus", QFilter.equals, HRBaseConstants.STR_ONE);
        QFilter enableFilter = new QFilter("enable", QFilter.equals, HRBaseConstants.ENABLED);
        QFilter initStatusFilter = new QFilter("initstatus", QFilter.not_in, new String[]{"0", "1"});
        DynamicObject[] aa = serviceHelper.loadDynamicObjectArray(
                new QFilter[]{idFilter, currentversionFilter, datastatusFilter, enableFilter, initStatusFilter});
        return aa;
    }

    /**
     * 获取职位的全量属性值
     *
     * @param ids
     * @return
     */
    public DynamicObject[] loadJobData(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{new QFilter(HRBaseConstants.ID, QFilter.in, ids)});
    }

    /**
     * 根据boid获取职位的全量属性值所有版本的数据
     *
     * @param ids
     * @return
     */
    public DynamicObject[] loadJobDataByBoid(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, ids);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{idFilter, QFilterHelper.getInitstatusQFilter()});
    }

    /**
     * 根据boid获取职位的全量属性值所有版本的数据
     *
     * @param ids
     * @return
     */
    public DynamicObject[] loadCurrentJobDataByBoid(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, ids);
        idFilter.and(new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ONE));
        idFilter.and(new QFilter(HBJMConstants.INIT_STATUS, QFilter.not_in,new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED}));
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        idFilter.and(enableFilter);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{idFilter, QFilterHelper.getInitstatusQFilter()});
    }

    /**
     * 根据boid获取职位的全量属性值所有版本的数据
     *
     * @param ids
     * @return
     */
    public DynamicObject[] loadCurrentJobFieldDataByBoid(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, ids);
        idFilter.and(new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ONE));
        idFilter.and(new QFilter(HBJMConstants.INIT_STATUS, QFilter.not_in,new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED}));
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        idFilter.and(enableFilter);
        String cols = "id,number,name,status,enable,createorg,useorg,ctrlstrategy,initstatus,boid,iscurrentversion,datastatus,bsed,jobscm," +
                "lowjoblevel,highjoblevel,lowjobgrade,highjobgrade,jobseq,jobfamily,jobclass";
        return serviceHelper.query(cols,new QFilter[]{idFilter});
    }

    /**
     * 根据boid、时间搜索职位的全量属性值历史版本的数据
     *
     * @param ids
     * @return
     */
    public DynamicObject[] loadHisJobFieldDataByBoId(List<Long> ids, Date searchDate) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, ids);
        idFilter.and(new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ZERO));
        idFilter.and(new QFilter(HisFieldNameConstants.DATA_VERSION_STATUS, QFilter.in,
                new String[]{EnumHisDataVersionStatus.INVALIDED.getStatus(),EnumHisDataVersionStatus.EFFECTING.getStatus()}));
        idFilter.and(new QFilter("bsed", QFilter.less_equals, searchDate));
        idFilter.and(new QFilter("bsled", QFilter.large_equals, searchDate));
        idFilter.and(new QFilter(HRBaseConstants.STATUS, QFilter.equals, HRBaseConstants.STATUS_AUDIT));
        idFilter.and(new QFilter(HBJMConstants.INIT_STATUS, QFilter.not_in,new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED}));
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        idFilter.and(enableFilter);
        String cols = "id,number,name,status,enable,createorg,useorg,ctrlstrategy,initstatus,boid,iscurrentversion,datastatus,bsed,jobscm," +
                "lowjoblevel,highjoblevel,lowjobgrade,highjobgrade,jobseq,jobfamily,jobclass";
        return serviceHelper.query(cols,new QFilter[]{idFilter});
    }


    /**
     * 获取禁用操作产生的历史版本
     *
     * @param ids
     * @return
     */
    public DynamicObject[] loadUnEnablingJobDataByBoid(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, ids);
        QFilter enableFilter = new QFilter("enable", QFilter.not_equals, HRBaseConstants.ENABLING);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{idFilter, enableFilter, QFilterHelper.getInitstatusQFilter()});
    }

    /**
     * 根据id查询数据
     *
     * @param ids
     * @return
     */
    public DynamicObject[] loadBaseData(List<Object> ids, String entityName) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(entityName);
        return serviceHelper.loadDynamicObjectArray(entityName, new QFilter[]{new QFilter(HRBaseConstants.ID, QFilter.in, ids)});
    }

    /**
     * 根据职位id查询最高最低职级职等
     *
     * @param ids ids
     * @return dys
     */
    public DynamicObject[] queryJobLevelGradeInfo(List<Long> ids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        return serviceHelper.query("id,jobscm,jobclass,lowjoblevel,highjoblevel,lowjobgrade,highjobgrade",
                new QFilter[]{new QFilter(HRBaseConstants.ID, QFilter.in, ids)});
    }

    /**
     * 根据职位id查询最高最低职级职等
     *
     * @param jobboidlist jobboid
     * @return dys
     */
    public DynamicObject[] queryJobLevelGrade(List<Long> jobboidlist) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter(HRBaseConstants.ID, QFilter.in, jobboidlist);
        return serviceHelper.query("id,lowjoblevel,lowjobgrade,highjoblevel,highjobgrade," +
                        "lowjoblevel.joblevelseq,lowjobgrade.jobgradeseq," +
                        "highjoblevel.joblevelseq,highjobgrade.jobgradeseq," +
                        "highjoblevel.joblevelscm,highjobgrade.jobgradescm",
                new QFilter[]{idFilter});


    }

    /**
     * 根据职位boid查询所有的数据
     *
     * @param  boids
     * @return dys
     */
    public DynamicObject[] queryDataByBoids(List<Long> boids) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter idFilter = new QFilter("boid", QFilter.in, boids);
        return serviceHelper.query("id,boid", new QFilter[]{idFilter});
    }

    /**
     * 根据职位boid查询所有的数据
     *
     * @param  qFilter
     * @return dys
     */
    public DynamicObject[] queryOriginalArray(QFilter qFilter) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        return serviceHelper.queryOriginalArray("id, boid, enable, name, number", new QFilter[]{qFilter});
    }

    public DynamicObject[] queryJobHisInfos(Collection<Long> jobIdList, Date earliestDate, Date latestDate) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        QFilter[] hisVersionQFilter = jobHisVersionQFilter(jobIdList, earliestDate, latestDate);
        String field = "id, boid, eanble, createorg, name, number, bsed, bsled";
        return serviceHelper.query(field, hisVersionQFilter);
    }

    private QFilter[] jobHisVersionQFilter(Collection<Long> jobIdList, Date earliestDate, Date latestDate) {
        QFilter boIdFilter = new QFilter(HisFieldNameConstants.BOID, QFilter.in, jobIdList);
        QFilter dataStatusFilter = new QFilter(HRHisBaseConstants.FIELD_DATASTATUS, QFilter.in,
                new String[] {HRHisBaseConstants.DATA_STATUS_EFFECTING, HRHisBaseConstants.DATA_STATUS_INVALID});
        QFilter isCurrentVersionFilter =
                new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QFilter.equals, HRBaseConstants.STR_ZERO);
        QFilter initStatusFilter = new QFilter(HBJMConstants.INIT_STATUS, QFilter.not_in, new String[] {"0", "1"});

        QFilter qFilter = boIdFilter.and(dataStatusFilter).and(isCurrentVersionFilter).and(initStatusFilter);
        if (earliestDate != null && latestDate != null) {
            QFilter earliestDateFilter =  new QFilter(HRHisBaseConstants.FIELD_BSLED, QFilter.large_equals, HRDateTimeUtils.truncateDate(earliestDate));
            QFilter latestDateFilter = new QFilter(HRHisBaseConstants.FIELD_BSED, QFilter.less_equals, HRDateTimeUtils.truncateDate(latestDate));
            earliestDateFilter.and(latestDateFilter);
            qFilter.and(earliestDateFilter);
        }

        return new QFilter[] {qFilter};
    }

    /**
     * 根据传进来的条件查询职位信息
     *
     * @param filterArray 过滤器数组
     * @return 数组
     */
    public DynamicObject[] loadDynamicArrayByFilters(QFilter[] filterArray) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_HBJM_JOB);
        return serviceHelper.loadDynamicObjectArray(filterArray);
    }

}
