package kd.hrmp.hbjm.business.domain.service.impl;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.OrmLocaleValue;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.db.DB;
import kd.bos.entity.BillEntityType;
import kd.bos.entity.EntityType;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.property.BasedataProp;
import kd.bos.entity.property.DateProp;
import kd.bos.entity.property.MulBasedataProp;
import kd.bos.entity.property.MuliLangTextProp;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.MetadataServiceHelper;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hr.hbp.common.util.HRDateTimeUtils;
import kd.hr.hbp.common.util.HRStringUtils;
import kd.hrmp.hbjm.business.domain.repository.HBJMEventRepository;
import kd.hrmp.hbjm.business.domain.service.ChangeEventService;
import kd.hrmp.hbjm.business.domain.service.ChangeMsgService;
import kd.hrmp.hbjm.common.constants.ChangeEventConstants;
import kd.hrmp.hbjm.common.util.MultiLangEqualsUtil;
import kd.hrmp.hbjm.common.constants.HBJMConstants;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 职位变动事务
 */
public class ChangeEventServiceImpl implements ChangeEventService {
    private Date now = new Date();
    private long creatorId = RequestContext.get().getCurrUserId();
    private Map<String, IDataEntityProperty> allFields = Maps.newHashMapWithExpectedSize(HRBaseConstants.INT_ONEHUNDRED);
    private String sourceType = "";
    private Map<Long, Long> afterVidToBillIdMap;

    @Override
    public void handleChangeEvent(String sourceType, DynamicObject[] beforeData, DynamicObject[] afterData,
                                  Map<Long, Long> afterVidToBillIdMap) {
        if (!checkParams(sourceType, afterData)) {
            return;
        }
        this.sourceType = sourceType;
        this.afterVidToBillIdMap = afterVidToBillIdMap;

        // 如果来源是职位，先获取职位上除分录外的所有字段(假设以后有分录)
        if (HRStringUtils.equals(sourceType, ChangeEventConstants.SOURCETYPE_JOB)) {
            initAllfieldsExcludeEntry(ChangeEventConstants.sourceTypeEntity.get(sourceType));
        }

        Map<Long, DynamicObject> beforeMap;
        if (beforeData == null) {
            beforeMap = Maps.newHashMapWithExpectedSize(HRBaseConstants.INT_EIGHT);
        } else {
            beforeMap = Arrays.stream(beforeData)
                .collect(Collectors.toMap(dynamicObject -> dynamicObject.getLong(HisFieldNameConstants.BOID),
                    dynamicObject -> dynamicObject, (v1, v2) -> v1));
        }

        long[] eventIds = DB.genGlobalLongIds(afterData.length);
        int index = 0;
        // 事务汇总
        List<DynamicObject> eventList = Lists.newArrayListWithExpectedSize(afterData.length);
        // 事务明细
        List<DynamicObject> eventDetailList = Lists.newArrayListWithExpectedSize(afterData.length);
        // 操作明细
        List<DynamicObject> opDetailList = Lists.newArrayListWithExpectedSize(afterData.length);

        //需要发送信息的集合
        List<DynamicObject> sendData = Lists.newArrayListWithExpectedSize(afterData.length);
        for (DynamicObject after : afterData) {

            // 新增待启用的岗位(只有在中台那边开启新增待启用的按钮才会产生这种数据)， 不发消息 不记事务
            if (HRStringUtils.equals(after.getString(HRBaseConstants.ENABLE), HRBaseConstants.ENABLING)) {
                continue;
            }
            long eventId = eventIds[index];
            index++;
            long boid = after.getLong(HisFieldNameConstants.BOID);
            DynamicObject before = beforeMap.get(boid);
            // 场景子类处理。事务汇总和事务明细的场景子类 值是一样的
            List<Long> subsceneIdList = handleChangeSubscene(after, before);

            // 先处理事务汇总表
            DynamicObject event = handleEventSum(after, eventId, subsceneIdList);
            eventList.add(event);

            // 处理事务明细
            DynamicObject eventDetail = handleEventDetal(after, eventId, subsceneIdList);
            eventDetailList.add(eventDetail);

            // 处理操作明细
            DynamicObject operDetail = handleOperDetail(after, before, eventId, sourceType);
            opDetailList.add(operDetail);

            // 排除职位初始化的数据（职位初始化的不需要发消息）
            if (HRStringUtils.equals(sourceType, ChangeEventConstants.SOURCETYPE_JOB)
                && HRStringUtils.equals(after.getString(HBJMConstants.INIT_STATUS), HRBaseConstants.STR_TWO)
                    && !HRStringUtils.equals(after.getString(HRBaseConstants.ENABLE), HRBaseConstants.ENABLING)
                    && HRStringUtils.equals(after.getString(HRBaseConstants.STATUS), HRBaseConstants.STATUS_AUDIT)) {
                sendData.add(after);
            }
        }

        HBJMEventRepository.getInstance().saveBatch(ChangeEventConstants.PAGE_EVENT_SUM, eventList);
        HBJMEventRepository.getInstance().saveBatch(ChangeEventConstants.PAGE_EVENT_DETAIL, eventDetailList);
        HBJMEventRepository.getInstance().saveBatch(ChangeEventConstants.PAGE_OP_DETAIL, opDetailList);

        ChangeMsgService msg = new ChangeMsgServiceImpl();
        msg.handleChangeMsg(eventList,sendData);
    }

    private boolean checkParams(String sourceType, DynamicObject[] afterData) {
        if (!ChangeEventConstants.sourceTypeEntity.containsKey(sourceType)) {
            return false;
        }

        if (afterData == null || afterData.length == 0) {
            return false;
        }
        return true;
    }

    private void initAllfieldsExcludeEntry(String entityPage) {
        MainEntityType entityType = MetadataServiceHelper.getDataEntityType(entityPage);
        Map<String, EntityType> entityTypeMap = entityType.getAllEntities();
        entityTypeMap.forEach((field, fieldType) -> {
            if (fieldType instanceof BillEntityType) {
                allFields.putAll(entityTypeMap.get(field).getFields());
            }
        });
    }

    private DynamicObject handleEventSum(DynamicObject after, long eventId, List<Long> subsceneIdList) {
        DynamicObject event = newEmptyDynamicObject(ChangeEventConstants.PAGE_EVENT_SUM);
        long id = createId(ChangeEventConstants.PAGE_EVENT_SUM);
        event.set("id", id);
        event.set("creator", creatorId);
        event.set("createtime", now);
        event.set("event", eventId);
        event.set("sourcetype", sourceType);
        event.set("targetjob", after.getLong(HisFieldNameConstants.BOID));
        event.set("changetype", after.getString("changetype"));
        event.set("changescene", after.getString("changescene"));
        event.set("changereason", after.getLong("changedesc.id"));
        event.set("changedate", after.getDate("bsed"));
        if (afterVidToBillIdMap != null) {
            event.set("billid", afterVidToBillIdMap.get(after.getLong(HRBaseConstants.ID)));
        }
        setSubsceneDynCol(event, subsceneIdList);
        // todo 关联职位 (目前的场景没有) 后期可以通过 单据id,找到此单的所有职位，但是要确定哪一个是最初的目标职位。
        return event;
    }

    private DynamicObject handleEventDetal(DynamicObject after, Long eventId, List<Long> subsceneIdList) {
        DynamicObject event = newEmptyDynamicObject(ChangeEventConstants.PAGE_EVENT_DETAIL);
        long id = createId(ChangeEventConstants.PAGE_EVENT_DETAIL);
        event.set("id", id);
        event.set("creator", creatorId);
        event.set("createtime", now);
        event.set("event", eventId);
        event.set("sourcetype", sourceType);
        event.set("targetjob", after.getLong(HisFieldNameConstants.BOID));
        event.set("changetype", after.getString("changetype"));
        event.set("changescene", after.getString("changescene"));
        event.set("changereason", after.getString("changedesc.id"));
        event.set("changedate", after.getDate("bsed"));
        setSubsceneDynCol(event, subsceneIdList);
        // todo 关联职位 (目前的场景没有)
        return event;
    }

    private DynamicObject handleOperDetail(DynamicObject after, DynamicObject before, Long eventId, String sourceType) {
        DynamicObject operDetail = newEmptyDynamicObject(ChangeEventConstants.PAGE_OP_DETAIL);
        long id = createId(ChangeEventConstants.PAGE_OP_DETAIL);
        operDetail.set("id", id);
        operDetail.set("creator", creatorId);
        operDetail.set("createtime", now);
        operDetail.set("event", eventId);
        operDetail.set("sourcetype", sourceType);
        operDetail.set("job", after.getLong(HisFieldNameConstants.BOID));
        operDetail.set("changeoperate", after.getLong("changeoperate"));
        operDetail.set("changedate", after.getDate("bsed"));
        operDetail.set("beforejob", (before == null || after.getLong("changeoperate") == ChangeEventConstants.ADD) ? 0L
            : before.getLong(HRBaseConstants.ID));
        operDetail.set("afterjob", after.getLong(HRBaseConstants.ID));
        return operDetail;
    }

    private List<Long> handleChangeSubscene(DynamicObject after, DynamicObject before) {
        // 变动场景子类多选。 只有 职位的变更操作 才可能有多选的情况
        List<Long> subsceneList = Lists.newArrayListWithExpectedSize(HRBaseConstants.INT_EIGHT);
        long operation = after.getLong("changeoperate");
        // 新增
        if (ChangeEventConstants.ADD == operation) {
            subsceneList.add(ChangeEventConstants.SUBSCENE_ADD);
        }
        // 禁用
        else if (ChangeEventConstants.DISABLE == operation) {
            subsceneList.add(ChangeEventConstants.SUBSCENE_DISABLE);
        }
        // 变更
        else if (ChangeEventConstants.UPDATE == operation) {
            // 来源是职位, 则对比判断有哪些场景子类
            if (HRStringUtils.equals(ChangeEventConstants.SOURCETYPE_JOB, sourceType)) {
                for (long subscene : ChangeEventConstants.SUBSCENE_COMPARE_FILED.keySet()) {
                    if (compareFieldsValueChange(after, before, subscene)) {
                        subsceneList.add(subscene);
                    }
                }
            }
        }
        // 没有值，那就填个信息变更吧
        if (subsceneList.size() == 0) {
            subsceneList.add(ChangeEventConstants.SUBSCENE_INFO_CHANGE);
        }
        return subsceneList;
    }

    private boolean compareFieldsValueChange(DynamicObject after, DynamicObject before, long subscene) {
        if (before == null) {
            return true;
        }
        // 需要对比的字段
        Set<String> needCompareFields = ChangeEventConstants.SUBSCENE_COMPARE_FILED.get(subscene);
        for (String fieldKey : needCompareFields) {
            IDataEntityProperty field = allFields.get(fieldKey);
            if (field instanceof MuliLangTextProp) {
                // 多语言字段对比
                OrmLocaleValue currentValue =  (OrmLocaleValue) after.get(field.getName());
                OrmLocaleValue oldValue =  (OrmLocaleValue) before.get(field.getName());
                if(!MultiLangEqualsUtil.compare(currentValue, oldValue, false)){
                    return true;
                }
            } else if (field instanceof MulBasedataProp) {
                // 多选基础资料判断
                DynamicObjectCollection afterMulBasedata = after.getDynamicObjectCollection(field.getName());
                DynamicObjectCollection beforeMulBasedata = before.getDynamicObjectCollection(field.getName());
                if (afterMulBasedata.size() != beforeMulBasedata.size()) {
                    return true;
                }
                List<String> afterMulBaseDataIds = Lists.newArrayListWithExpectedSize(afterMulBasedata.size());
                List<String> beforeMulBaseDataIds = Lists.newArrayListWithExpectedSize(afterMulBasedata.size());
                for (int index = 0; index < afterMulBasedata.size(); index++) {
                    afterMulBaseDataIds.add(afterMulBasedata.get(index).getString("fbasedataid_id"));
                    beforeMulBaseDataIds.add(beforeMulBasedata.get(index).getString("fbasedataid_id"));
                }
                afterMulBaseDataIds.removeAll(beforeMulBaseDataIds);
                if (afterMulBaseDataIds.size() > 0) {
                    return true;
                }
            } else if (field instanceof DateProp) {
                if (!HRDateTimeUtils.format(after.getDate(field.getName()))
                    .equals(HRDateTimeUtils.format(before.getDate(field.getName())))) {
                    return true;
                }
            } else if (field instanceof BasedataProp) {
                if (!HRStringUtils.equals(after.getString(field.getName() + ".id"), before.getString(field.getName() + ".id"))) {
                    return true;
                }
            } else if (!HRStringUtils.equals(after.getString(field.getName()), before.getString(field.getName()))) {
                return true;
            }
        }
        return false;
    }

    private void setSubsceneDynCol(DynamicObject dyn, List<Long> subsceneIds) {
        DynamicObjectCollection collection = new DynamicObjectCollection();
        DynamicObjectType dynamicObjectType = dyn.getDynamicObjectCollection("changesubscene").getDynamicObjectType();
        for (Long subsceneId : subsceneIds) {
            DynamicObject subcene = new DynamicObject(dynamicObjectType);
            subcene.set("fbasedataid", subsceneId);
            collection.add(subcene);
        }
        if (collection.size() > 0) {
            dyn.set("changesubscene", collection);
        }
    }

    private static long createId(String pageValue) {
        return ORM.create().genLongId(pageValue);
    }

    private DynamicObject newEmptyDynamicObject(String pageValue) {
        MainEntityType dataEntityType = MetadataServiceHelper.getDataEntityType(pageValue);
        return new DynamicObject(dataEntityType);
    }

    @Override
    public void delEvent(List<Long> jobIds) {
        HRBaseServiceHelper eventsumHelper = new HRBaseServiceHelper(ChangeEventConstants.PAGE_EVENT_SUM);
        DynamicObject[] eventsums =
            eventsumHelper.queryOriginalArray("id,event", new QFilter[] {new QFilter("targetjob", QFilter.in, jobIds)});

        Set<Long> eventIds =
            Arrays.stream(eventsums).map(eventsum -> eventsum.getLong("event")).collect(Collectors.toSet());

        HBJMEventRepository.getInstance().delEventSum(eventIds);
        HBJMEventRepository.getInstance().delEventDetail(eventIds);
        HBJMEventRepository.getInstance().delOpDetail(eventIds);
    }
}
